/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomStyleSheet.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomStyleSheet.h"
#include "gdom/PrivateIntGdomStyleSheet.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMStyleSheet::GDOMStyleSheet(PassRefPtr<WebCore::StyleSheet> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMStyleSheet::~GDOMStyleSheet()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "StyleSheet.h"
#include <gdom/GdomStyleSheet.h>
#include <gdom/PrivateGdomStyleSheet.h>
#include <gdom/PrivateIntGdomStyleSheet.h>
#include <gdom/GdomStyleSheetprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomStyleSheet, gdom_style_sheet, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomStyleSheetPrivate {
    class WebCore::StyleSheet *item;
};

#define GDOM_STYLE_SHEET_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_STYLE_SHEET, GdomStyleSheetPrivate))
*/
G_END_DECLS
WebCore::StyleSheet *
gdom_style_sheet_get_private(GdomStyleSheet *request)
{
    GDOMStyleSheet *res;
    res = static_cast<GDOMStyleSheet*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::StyleSheet*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_TYPE,
    PROP_DISABLED,
    PROP_OWNER_NODE,
    PROP_PARENT_STYLE_SHEET,
    PROP_HREF,
    PROP_TITLE,
    PROP_MEDIA,
};


static void gdom_style_sheet_finalize(GObject* object)
{
    /*GdomStyleSheet* request = GDOM_STYLE_SHEET(object);*/
    /*GdomStyleSheetPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_style_sheet_parent_class)->finalize(object);
}

static void gdom_style_sheet__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomStyleSheet *self = GDOM_STYLE_SHEET(object);
    WebCore::StyleSheet * gself = (gdom_style_sheet_get_private(self));
    switch (prop_id) {
     case PROP_DISABLED:
    {
         gself->setDisabled((g_value_get_boolean (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_style_sheet__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomStyleSheet *self = GDOM_STYLE_SHEET(object);
    WebCore::StyleSheet * gself = (gdom_style_sheet_get_private(self));
    switch (prop_id) {
    case PROP_TYPE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->type()));
         break;
    }
    case PROP_DISABLED:
    {
         g_value_set_boolean (value, gself->disabled());
         break;
    }
    case PROP_OWNER_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->ownerNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PARENT_STYLE_SHEET:
    {
         PassRefPtr<WebCore::StyleSheet> ptr = gself->parentStyleSheet();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_HREF:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->href()));
         break;
    }
    case PROP_TITLE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->title()));
         break;
    }
    case PROP_MEDIA:
    {
         PassRefPtr<WebCore::MediaList> ptr = gself->media();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_style_sheet_class_init(GdomStyleSheetClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_style_sheet_finalize;
    g_klass->set_property = gdom_style_sheet__set_property_fn;
    g_klass->get_property = gdom_style_sheet__get_property_fn;

    g_object_class_install_property (g_klass, PROP_TYPE,
            g_param_spec_string ("type", /* name */
                "style_sheet_type", /* short description */
                "read-only  gchar * StyleSheet.type", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DISABLED,
            g_param_spec_boolean ("disabled", /* name */
                "style_sheet_disabled", /* short description */
                "read-write  gboolean StyleSheet.disabled", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_OWNER_NODE,
            g_param_spec_object ("owner_node", /* name */
                "style_sheet_owner_node", /* short description */
                "read-only  GdomNode * StyleSheet.owner_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PARENT_STYLE_SHEET,
            g_param_spec_object ("parent_style_sheet", /* name */
                "style_sheet_parent_style_sheet", /* short description */
                "read-only  GdomStyleSheet * StyleSheet.parent_style_sheet", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_STYLE_SHEET, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_HREF,
            g_param_spec_string ("href", /* name */
                "style_sheet_href", /* short description */
                "read-only  gchar * StyleSheet.href", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_TITLE,
            g_param_spec_string ("title", /* name */
                "style_sheet_title", /* short description */
                "read-only  gchar * StyleSheet.title", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_MEDIA,
            g_param_spec_object ("media", /* name */
                "style_sheet_media", /* short description */
                "read-only  GdomMediaList * StyleSheet.media", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_MEDIA_LIST, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomStyleSheetPrivate));*/
}

static void gdom_style_sheet_init(GdomStyleSheet* request)
{
    /*GdomStyleSheetPrivate* priv = GDOM_STYLE_SHEET_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

